@@ -70,7 +70,9 @@ enum dump_format {
 \
     0, 0, 0, 0, 0, 0, 0, \
 \
-    0, 0, 0, 0, 0 \
+    0, 0, 0, 0, 0, \
+\
+    0 \
 }
 
 /* clang-format on */
@@ -109,6 +111,8 @@ struct options {
     int    reassemble_tcp;
     size_t reassemble_tcp_faultreset;
     int    reassemble_tcp_bfbparsedns;
+
+    int bpf_hosts_apply_all;
 };
 
 int option_parse(options_t* options, const char* option);
